'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.download = exports.setMetadata = exports.fetchFromYoutube = exports.fetchFromiTunes = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _http = require('http');

var _http2 = _interopRequireDefault(_http);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _nodeId = require('node-id3');

var _nodeId2 = _interopRequireDefault(_nodeId);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _untildify = require('untildify');

var _untildify2 = _interopRequireDefault(_untildify);

var _youtubeNode = require('youtube-node');

var _youtubeNode2 = _interopRequireDefault(_youtubeNode);

var _itunesInfo = require('itunes-info');

var _youtubeDlStatus = require('youtube-dl-status');

var _youtubeDlStatus2 = _interopRequireDefault(_youtubeDlStatus);

var _spinner = require('./spinner');

var _spinner2 = _interopRequireDefault(_spinner);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var timeStrToMillis = function timeStrToMillis(str) {
  var units = { S: 1000, M: 1000 * 60, H: 1000 * 60 * 60, D: 1000 * 60 * 60 * 24 };
  var content = str.replace('PT', '');
  var numbers = content.split(/\D/g).filter(function (e) {
    return !!e;
  });
  var symbols = content.match(/\D/g).filter(function (e) {
    return !!e;
  });
  return numbers.reduce(function (a, b, i) {
    return a + b * units[symbols[i]];
  }, 0);
};

var expandYoutube = function expandYoutube(youtube, element) {
  return new _bluebird2.default(function (resolve, reject) {
    youtube.getById(element.id.videoId, function (error, result) {
      if (error) {
        return reject(error);
      }
      if (result.items.length < 1 || !result.items[0]) {
        return resolve(undefined);
      }
      var video = result.items[0];
      video.link = 'https://www.youtube.com/watch?v=' + video.id;
      video.durationMillis = timeStrToMillis(video.contentDetails.duration);
      return resolve(video);
    });
  });
};

var fetchFromiTunes = exports.fetchFromiTunes = function fetchFromiTunes(options) {
  return _itunesInfo.iTunes.fetch(options.query).then(function (data) {
    if (options.verbose) {
      _spinner2.default.text = 'Fetching iTunes data...';
    }
    if (data.results.length < 1) {
      throw new Error('No results!');
    }
    var tracks = data.results.filter(function (res) {
      return res.kind === 'song';
    });
    if (tracks.length < 1) {
      throw new Error('No tracks!');
    }
    tracks[0].artworkUrl512 = tracks[0].artworkUrl100.replace('100x100', '512x512');
    tracks[0].filename = tracks[0].artistName + ' - ' + tracks[0].trackCensoredName;
    tracks[0].path = {
      file: _path2.default.resolve((0, _untildify2.default)(options.directory), tracks[0].filename + '.mp3'),
      image: _path2.default.resolve((0, _untildify2.default)(options.directory), tracks[0].filename + '.jpg')
    };
    return tracks[0];
  });
};

var fetchFromYoutube = exports.fetchFromYoutube = function fetchFromYoutube(options) {
  return new _bluebird2.default(function (resolve, reject) {
    if (options.verbose) {
      _spinner2.default.text = 'Finding Youtube match...';
    }
    var youtube = new _youtubeNode2.default();
    youtube.setKey(options.token);
    youtube.search(options.filename, options.results, function (error, results) {
      if (error) {
        return reject(new Error('Bad request, make sure you have a valid token!'));
      }
      if (results.items.length === 0) {
        return reject(new Error('No YouTube results!'));
      }
      return _bluebird2.default.map(results.items, function (res) {
        return expandYoutube(youtube, res);
      }, { concurrency: 50 }).then(function (videos) {
        var list = videos.filter(function (v) {
          return !!v;
        });
        list.forEach(function (res) {
          res.timeDifference = Math.abs(res.durationMillis - options.trackTimeMillis);
        });
        list.sort(function (a, b) {
          return a.timeDifference - b.timeDifference;
        });
        resolve(_extends({}, options, list[0]));
      });
    });
  });
};

var setMetadata = exports.setMetadata = function setMetadata(options) {
  return new _bluebird2.default(function (resolve, reject) {
    if (options.verbose) {
      _spinner2.default.text = 'Writing metadata...';
    }
    _http2.default.get(options.artworkUrl512, function (response) {
      if (response.statusCode === 200) {
        response.pipe(_fs2.default.createWriteStream(options.path.image)).on('finish', function () {
          _nodeId2.default.removeTags(options.path.file);
          var success = _nodeId2.default.write({
            title: options.trackCensoredName,
            artist: options.artistName,
            album: options.collectionCensoredName,
            image: options.path.image,
            trackNumber: options.trackNumber,
            genre: options.primaryGenreName,
            date: options.releaseDate,
            year: options.releaseDate.substring(0, 4)
          }, options.path.file);
          if (!success) {
            return reject(new Error('Unable to write metadata!'));
          }
          return _fs2.default.unlink(options.path.image, resolve);
        });
      } else {
        reject(new Error('Unable to fetch metadata!'));
      }
    });
  });
};

var download = exports.download = function download(options) {
  return new _bluebird2.default(function (resolve, reject) {
    if (options.verbose) {
      _spinner2.default.text = 'Starting download...';
    }
    var conf = ['--extract-audio', '--audio-quality=0', '--format=bestaudio', '--audio-format=mp3', '--output=' + _path2.default.resolve((0, _untildify2.default)(options.directory), options.filename + '.%(ext)s')];
    var downloadProcess = _youtubeDlStatus2.default.exec(options.link, conf, function () {
      setMetadata(options).then(resolve).catch(reject);
    });
    downloadProcess.stdout.on('data', function (data) {
      var re = /[0-9]+((\.[0-9]{1}){0,1})%/i;
      var matches = data.match(re);
      if (matches && matches.length && matches.length > 0) {
        var percentage = matches[0];
        var text = percentage !== '100%' ? 'Downloading track... ' + percentage : 'Converting...';
        if (options.verbose) {
          _spinner2.default.text = text;
        }
      }
    });
  });
};